﻿using System.Linq;
using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using DataContracts;
using Ato.EN.IntegrationServices.CodeGenerationIEE;
using Ato.EN.IntegrationServices.CodeGenerationPTR;

namespace Ato.CD.Inbound.PTR201802
{
    internal class CrossFormValidatorIEE : ICrossFormValidator
    {

        private PTR2018 ParentDocument { get; }
        private IEE2018 ChildDocument { get; }

        internal CrossFormValidatorIEE(PTR2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (IEE2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();         
            VRATOIEE410223(response);//
            VRATOIEE410225(response);//
            VRATOGEN410115(response);//
            VRATOIEE410226(response);//
            VRATOGEN410119(response);//
            VRATOGEN402010(response);//
            VRATOIEE410198(response);//
            VRATOIEE410071(response);//
            VRATOIEE410193(response);//
            VRATOIEE410069(response);//
            VRATOIEE410076(response);//
            VRATOIEE410100(response);//
            VRATOIEE410070(response);//
            VRATOIEE410084(response);//
            VRATOIEE410051(response);//
            VRATOIEE410262(response);//
            VRATOPTR430176(response);//
            VRATOGEN430008(response);//
            VRATOPTR430178(response);       
            return response;
        }

        #region VR.ATO.IEE.410223
        /* 
        WHERE PARENT RETURN EXISTS
        IF ([IEE5] <> NULLORBLANK) AND ([IEE5] <> (PARENT RETURN:RP:pyde.xx.xx:AddressDetails.Line1.Text WHERE ((TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Usage.Code = "POS") AND (TUPLE ELEMENT EXPLICIT pyde.xx.xx:AddressDetails.Currency.Code = "C")) IN TUPLE(xbrli\address2.xx.xx:AddressDetails)))
           RETURN VALIDATION MESSAGE
        ENDIF
        [IEE5] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = "POS" AND pyde.02.00:AddressDetails.Currency.Code = "C") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails)
        */
        private void VRATOIEE410223(List<ProcessMessageDocument> response)
        {
            bool assertion = !(string.IsNullOrWhiteSpace(ChildDocument.IEE5)) && ChildDocument.IEE5 != ParentDocument.PTR9;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410223",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Interposed Entity Address must be blank or be the same as on main form",
                    LongDescription = "When the IEE is lodged as a schedule, the Address of the interposed entity must be either blank or must equal the address on the main form",
                    Location = "/xbrli:xbrl/tns:AddressDetails/tns:AddressDetails.Line1.Text",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410223" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410225
        /*
        WHERE PARENT RETURN EXISTS
        IF([IEE10] <> NULLORBLANK) AND([IEE10] <> (PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text)
                IN TUPLE(xbrli\orgname2.xx.xx:OrganisationNameDetails) 
                WHERE((TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = ""MN"") 
                AND (TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.Currency.Code = ""P""))) 
            RETURN VALIDATION MESSAGE
        ENDIF
        [IEE10] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE(pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = ""MN"" AND pyde.02.00:OrganisationNameDetails.Currency.Code = ""P"") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails)

        */
        private void VRATOIEE410225(List<ProcessMessageDocument> response)
        {

            bool assertion = !(string.IsNullOrWhiteSpace(ChildDocument.IEE10)) && ChildDocument.IEE10 != ParentDocument.PTR8;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Name on schedule does not match parent form",
                    LongDescription = "Name on schedule does not match name on parent form.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410225" } }
                });
            }
        }
        #endregion

        #region VR.ATO.GEN.410115
        /*
        WHERE PARENT RETURN EXISTS 
        IF (IEE:RP:entity.identifier.TFN <> NULLORBLANK) 
        AND (IEE:RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
               RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATOGEN410115(List<ProcessMessageDocument> response)
        {
            bool assertion = !(string.IsNullOrWhiteSpace(ChildDocument.IEE3)) && ChildDocument.IEE3 != ParentDocument.PTR5;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410115",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "TFN on Schedule must match TFN on main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410115" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410226
        /*       
         WHERE PARENT RETURN EXISTS
         IF ([IEE4] <> NULLORBLANK) AND 
         ([IEE4] <> (PARENT RETURN:RP:pyde.xx.xx:OrganisationNameDetails.OrganisationalName.Text 
         IN TUPLE (xbrli\orgname2.xx.xx:OrganisationNameDetails) 
         WHERE ((TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.OrganisationalNameType.Code = ""MN"") 
         AND (TUPLE EXPLICIT pyde.xx.xx:OrganisationNameDetails.Currency.Code = ""C""))))
         RETURN VALIDATION MESSAGE
         ENDIF
         [IEE4] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = ""MN"" AND pyde.02.00:OrganisationNameDetails.Currency.Code = ""C"") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails)
        */
        private void VRATOIEE410226(List<ProcessMessageDocument> response)
        {
            bool assertion = !(string.IsNullOrWhiteSpace(ChildDocument.IEE4)) && ChildDocument.IEE4 != ParentDocument.PTR6;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Name on schedule does not match parent form",
                    LongDescription = "Name on schedule does not match name on parent form.",
                    Location = "/xbrli:xbrl/tns::OrganisationNameDetails/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410226" } }
                });
            }
        }
        #endregion

        #region VR.ATO.GEN.410119
        /*
        WHERE PARENT RETURN EXISTS
        IF (IEE:INT:entity.identifier.ABN <> NULLORBLANK) AND (IEE:INT:entity.identifier.ABN <> 
        PARENT RETURN:INT:entity.identifier.ABN)
              RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATOGEN410119(List<ProcessMessageDocument> response)
        {
            bool assertion = !(string.IsNullOrWhiteSpace(ChildDocument.INTIdentifierABN)) && ChildDocument.INTIdentifierABN != ParentDocument.INTIdentifierABN;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410119",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "ABN of the Intermediary on the schedule must match the ABN on the main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410119" } }
                });
            }
        }
        #endregion

        #region VR.ATO.GEN.402010
        /*
        IF (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) 
        AND (PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) 
        AND (RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier 
        <> PARENT RETURN:RP:pyid.xx.xx:Identifiers.AustralianBusinessNumber.Identifier)
        RETURN VALIDATION MESSAGE
        ENDIF
        */
        private void VRATOGEN402010(List<ProcessMessageDocument> response)
        {
            bool assertion = !(string.IsNullOrWhiteSpace(ChildDocument.IEE162))
                          && !(string.IsNullOrWhiteSpace(ParentDocument.PTR7))
                          && ChildDocument.IEE162 != ParentDocument.PTR7;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = "Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402010" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410198
        /*
            WHERE PARENT RETURN EXISTS
            IF ([IEE116] = ""E"") AND ([IEE62] = NULLORBLANK) AND ([IEE63] <> TRUE) 
               RETURN VALIDATION MESSAGE
            ENDIF
            [IEE62] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE63] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumberInexistent.Indicator IN TUPLE(iee.0003.lodge.req.02.00:SpecifiedIndividual)
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        */
        private void VRATOIEE410198(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE116 == "E" && string.IsNullOrWhiteSpace(ChildDocument.IEE62) && ChildDocument.IEE63 != true;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "TFN of specified individual must be present if TFN non existent indicator is not 'TRUE'",
                    LongDescription = "TFN of specified individual must be present if TFN non existent indicator is blank or 'FALSE'",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410198" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410071
        /*
        WHERE PARENT RETURN EXISTS
        IF ([IEE116] = ""R"") AND 
        (
            ([IEE52] = NULLORBLANK) OR ([IEE53] = NULLORBLANK) 
            OR
            ([IEE54] = NULLORBLANK) OR ([IEE119] = NULLORBLANK) 
            OR
            ([IEE170] = NULLORBLANK) OR ([IEE82] = NULLORBLANK)
        )
           RETURN VALIDATION MESSAGE
        ENDIF

        [IEE52] = IEE:RP:pyde.02.00:OrganisationNameDetails.OrganisationalName.Text WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = ""MN"" AND pyde.02.00:OrganisationNameDetails.Currency.Code = ""C"") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
        [IEE53] = IEE:RP:pyde.02.00:AddressDetails.Line1.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = ""POS"" AND pyde.02.00:AddressDetails.Currency.Code = ""C"") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
        [IEE54] = IEE:RP:pyde.02.00:AddressDetails.LocalityName.Text WHERE (pyde.02.01:AddressDetails.Usage.Code = ""POS"" AND pyde.02.00:AddressDetails.Currency.Code = ""C"") IN TUPLE(iee.0003.lodge.req.02.00:AddressDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
        [IEE82] = IEE:RP:rvctc3.02.00:Elections.Revocation.Year
        [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        [IEE119] = IEE:RP:pyde.02.18:OrganisationNameDetails.OrganisationalNameType.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = ""MN"" AND pyde.02.00:OrganisationNameDetails.Currency.Code = ""C"") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
        [IEE170] = IEE:RP:pyde.02.00:OrganisationNameDetails.Currency.Code WHERE (pyde.02.00:OrganisationNameDetails.OrganisationalNameType.Code = ""MN"" AND pyde.02.00:OrganisationNameDetails.Currency.Code = ""C"") IN TUPLE(iee.0003.lodge.req.02.00:OrganisationNameDetails) IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
        */
        private void VRATOIEE410071(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE116 == "R"
                &&
                (
                    string.IsNullOrWhiteSpace(ChildDocument.IEE52) ||
                    string.IsNullOrWhiteSpace(ChildDocument.IEE53) ||
                    string.IsNullOrWhiteSpace(ChildDocument.IEE54) ||
                    string.IsNullOrWhiteSpace(ChildDocument.IEE119) ||
                    string.IsNullOrWhiteSpace(ChildDocument.IEE170) ||
                    string.IsNullOrWhiteSpace(ChildDocument.IEE82.ToString())
                );

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410071",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "For a revocation all details about the family trust must be present",
                    LongDescription = "You have nominated to revoke an interposed entity, however information is missing from the revocation section on the schedule. For the revocation to be valid Section B on the IEE schedule must be completed",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionOrRevocation.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410071" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410193
        /*
           WHERE PARENT RETURN EXISTS
           IF ([IEE116] = ""R"") AND ([IEE50] = NULLORBLANK) 
           RETURN VALIDATION MESSAGE
           ENDIF
           [IEE50] = IEE:RP:pyid.02.00:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0003.lodge.req.02.00:FamilyTrust)
           [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        */
        private void VRATOIEE410193(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE116 == "R" && string.IsNullOrWhiteSpace(ChildDocument.IEE50);

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410193",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "For a revocation, Family Trust tax file number must be present",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410193" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410069
        /*
            WHERE PARENT RETURN EXISTS
            IF ([IEE82] <> NULLORBLANK) AND ([IEE116] = ""E"")
                RETURN VALIDATION MESSAGE
            ENDIF

            [IEE82] = IEE:RP:rvctc3.02.00:Elections.Revocation.Year
            [IEE116] = IEE:RP:rvctc3.02.02:Elections.InterposedEntityElectionOrRevocation.Code
        */
        private void VRATOIEE410069(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE82.HasValue && ChildDocument.IEE116.ToUpper().Equals("E");
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Questions about revocation must not be answered if the form is used for an election",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410069" },
                        new ProcessMessageParameter { Name = "IEE82", Value = ChildDocument.IEE82.Value.ToString() }
                    }
                });
            }
        }       
        #endregion
        
        #region VR.ATO.IEE.410076
        //WHERE PARENT RETURN EXISTS
        //IF(([IEE58] <> NULLORBLANK) OR([IEE59] <> NULL) OR([IEE60] <> NULL) OR([IEE61] <> NULL) 
        //OR([IEE62] <> NULLORBLANK) OR([IEE69] <> NULLORBLANK) OR([IEE64] <> NULL) OR([IEE65] <> NULLORBLANK) 
        //OR([IEE66] <> NULLORBLANK) OR([IEE67] <> NULLORBLANK) OR([IEE68] <> NULLORBLANK) OR([IEE69] <> NULLORBLANK) 
        //OR([IEE70] <> NULLORBLANK) OR([IEE71] <> NULLORBLANK) OR([IEE72] <> NULLORBLANK) OR([IEE73] <> NULLORBLANK) 
        //AND([IEE82] <> NULLORBLANK)
        //RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410076(List<ProcessMessageDocument> response)
        {
            bool assertion = (ChildDocument.IEE58.HasValue || ChildDocument.IEE59.HasValue ||
                              ChildDocument.IEE60.HasValue || ChildDocument.IEE61.HasValue ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE62) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE69) || ChildDocument.IEE64.HasValue ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE65) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE66) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE67) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE68) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE69) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE70) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE71) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE72) ||
                              !string.IsNullOrWhiteSpace(ChildDocument.IEE73)) && ChildDocument.IEE82.HasValue;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410076",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "An IEE election must not contain revocation Income Year",
                    LongDescription = "This schedule contains election details, and also the 'income year from which the revocation is to be effective'. If both an election and a revocation are needed, a second schedule is to be submitted with the main return form",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410076" },
                        new ProcessMessageParameter { Name = "IEE82", Value = ChildDocument.IEE82.ToString() }
                    }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410100
        //WHERE PARENT RETURN EXISTS
        //IF([IEE116] = "E") AND([IEE16] = SET("217", "212", "161")) AND([IEE26] <> TRUE) AND([IEE25] = NULLORBLANK)
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410100(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE116.ToUpper().Equals("E") &&
                             !string.IsNullOrWhiteSpace(ChildDocument.IEE16) &&
                             new[] { "217", "212", "161" }.Any(s => ChildDocument.IEE16.Equals(s)) &&
                             ChildDocument.TrusteeOrPartnerCollectionExists &&
                             ChildDocument.TrusteeOrPartnerCollection.All(i => i.IEE26 == false) &&
                             ChildDocument.TrusteeOrPartnerCollection.All(i => string.IsNullOrWhiteSpace(i.IEE25));
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410100",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "The TFN of the trustee or partner or the no TFN indicator must be present",
                    LongDescription = "The interposed entity code is '161', '217' or '212' and the field indicating that the 'trustee or partner does not have a tax file number' is blank, and no tax file number has been supplied",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410100" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410070
        //WHERE PARENT RETURN EXISTS
        //IF([IEE116] = "R") AND([IEE82] <> PARENT RETURN:Report.TargetFinancial.Year)
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410070(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE116.ToUpper().Equals("R") && ChildDocument.IEE82.HasValue &&
                             ChildDocument.IEE82 != ParentDocument.PTR1;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410070",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "The income year of the Interposed Entity Revocation must equal the income year of the main return",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410070" },
                        new ProcessMessageParameter { Name = "IEE82", Value = ChildDocument.IEE82.Value.ToString() }
                    }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410084
        //WHERE PARENT RETURN EXISTS
        //IF([IEE82] <> NULLORBLANK) AND([IEE82] < "2011")
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410084(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE82.HasValue && ChildDocument.IEE82 < 2011;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410238",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Year specified for IEE revocation must be 2011 or later",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410084" },
                        new ProcessMessageParameter { Name = "IEE82", Value = ChildDocument.IEE82.Value.ToString() }
                    }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410262
        //WHERE PARENT RETURN EXISTS
        //IF INT<> NULLORBLANK AND COUNT[IEE159] > 0
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410262(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.INTExists && ChildDocument.IEE159 != null;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410262",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "If IEE is lodged as a schedule intermediary declaration must not be present for IEE",
                    Location = "/xbrli:xbrl/tns:Declaration/tns:Declaration.StatementType.Code",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410262" } }
                });
            }
        }
        #endregion

        #region VR.ATO.IEE.410051
        //WHERE PARENT RETURN EXISTS
        //IF([IEE49] <> NULL) AND([IEE45] <> PARENT RETURN:Report.TargetFinancial.Year)
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410051(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE49.HasValue && ChildDocument.IEE45 != ParentDocument.PTR1;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410235",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Election Commencement Time must not be present if Income Year Specified is different to the parent return target financial year",
                    Location = "/xbrli:xbrl/tns:Elections.Commencement.Date",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410051" },
                        new ProcessMessageParameter { Name = "IEE49", Value = ChildDocument.IEE49.Value.ToString() }
                    }
                });
            }
        }
        #endregion

        #region VR.ATO.PTR.430176
        /*
                Rule ID: VR.ATO.PTR.430176

                IF(PTR:RP:pyin.xx.xx:Report.TargetFinancial.Year<> NULL)
                  AND(COUNT(SCHEDULE = ""IEE"") > 0) 
                  AND(ANY OCCURRENCE OF ([IEE116] = ""R"")
                  AND([IEE82] <> PTR:RP:pyin.xx.xx:Report.TargetFinancial.Year))
                      RETURN VALIDATION MESSAGE
                ENDIF

                [IEE82] = IEE:RP:rvctc3.xx.xx:Elections.Revocation.Year
                [IEE116] = IEE:RP:rvctc3.xx.xx:Elections.InterposedEntityElectionOrRevocation.Code
        */
        private void VRATOPTR430176(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.PTR1.HasValue && !string.IsNullOrWhiteSpace(ChildDocument.IEE116) && ChildDocument.IEE116 == "R"
                && ChildDocument.IEE82.HasValue && ChildDocument.IEE82 != ParentDocument.PTR1;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430176",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Year of revocation must be the same as the year of the return",
                    LongDescription = "The income year for which the details of the Interposed Entity Election are being revoked must equal the income year of the main return being lodged.",
                    Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.PTR.430176" },
                        new ProcessMessageParameter { Name = "IEE82", Value = ChildDocument.IEE82.Value.ToString() }
                    }
                });
            }
        }
        #endregion

        #region VR.ATO.GEN.430008
        /*
        Rule ID: VR.ATO.GEN.430008

        IF(COUNT(SCHEDULE = ""IEE"") > 0) AND RP:entity.identifier.TFN = (ANY OCCURRENCE OF([IEE50])) 
            RETURN VALIDATION MESSAGE
        ENDIF

        [IEE50] = IEE:RP:pyid.xx.xx:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.xx.xx:FamilyTrust)
        [IEE50] = IEE:RP:pyid.xx.xx:Identifiers.TaxFileNumber.Identifier IN TUPLE(iee.0002.lodge.req.xx.xx:FamilyTrust)
*/
        private void VRATOGEN430008(List<ProcessMessageDocument> response)
        {
            bool assertion = ParentDocument.RPIdentifierTFN == ChildDocument.IEE50;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430008",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "The Family Trust TFN on the IEE must not be the same as the main entity TFN",
                    LongDescription = "An Entity cannot interpose itself; therefore the family trust tax file number quoted in Interposed Entity Election cannot be the same as the tax file number of the entity making the election",
                    Location = "/xbrli:xbrl/tns:Identifiers.TaxFileNumber.Identifier",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430008" },
                        new ProcessMessageParameter { Name = "IEE50", Value = ChildDocument.IEE50.ToString() }
                    }
                });
            }
        }
        #endregion

        #region VR.ATO.PTR.430178
        /*
        Rule ID: VR.ATO.PTR.430178
        
        IF[PTR26] = NULL AND(COUNT(SCHEDULE = ""IEE"") > 0)
            RETURN VALIDATION MESSAGE
        ENDIF

        [PTR26] = PTR:RP:Elections.InterposedEntityElectionStatus.Year
            */
        private void VRATOPTR430178(List<ProcessMessageDocument> response)
        {
            bool assertion = !ParentDocument.PTR26.HasValue;

            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.430178",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "IEE specified income year must be present",
                    LongDescription = "If an IEE schedule is present, the Interposed entity election status - Specified income year must be present",
                    Location = "/xbrli:xbrl/tns:Elections.InterposedEntityElectionStatus.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.430178" }                        
                    }
                });
            }
        }
        #endregion       
    }
}