﻿using System.Collections.Generic;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationIEE;
using Ato.EN.IntegrationServices.CodeGenerationSMSFAR;
using DataContracts;

namespace Ato.CD.Inbound.SMSFAR201802
{
    internal class CrossFormValidatorIEE : ICrossFormValidator
    {
        private SMSFAR2018 ParentDocument { get; }

        private IEE2018 ChildDocument { get; }

        internal CrossFormValidatorIEE(SMSFAR2018 report, BusinessDocument childDocument)
        {
            ParentDocument = report;
            ChildDocument = (IEE2018)childDocument.ConsumedReport;
        }

        public IEnumerable<ProcessMessageDocument> ValidateCrossFormRules()
        {
            var response = new List<ProcessMessageDocument>();

            VRATOGEN402010(response);
            VRATOGEN410115(response);
            VRATOGEN410119(response);
            VRATOIEE410051(response);
            VRATOIEE410070(response);
            VRATOIEE410223(response);
            VRATOIEE410226(response);

            return response;
        }

        #region VR.ATO.GEN.402010
        //IF (RP:tns:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) 
        //AND (PARENT RETURN:RP:tns:Identifiers.AustralianBusinessNumber.Identifier <> NULLORBLANK) 
        //AND (RP:tns:Identifiers.AustralianBusinessNumber.Identifier <> PARENT RETURN:RP:tns:Identifiers.AustralianBusinessNumber.Identifier)
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOGEN402010(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ParentDocument.SMSFAR5) &&
                             !string.IsNullOrWhiteSpace(ChildDocument.IEE162) &&
                             !ParentDocument.SMSFAR5.Equals(ChildDocument.IEE162);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402010",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Your supplied ABN does not match the ABN supplied on the form it was submitted with",
                    LongDescription = "Your supplied ABN does not match the ABN supplied on the parent form it was submitted with",
                    Location = "/xbrli:xbrl/tns:Identifiers.AustralianBusinessNumber.Identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402010" } }
                });
            }
        }
        #endregion
        #region VR.ATO.GEN.410115
        //WHERE PARENT RETURN EXISTS 
        //IF(IEE:RP:entity.identifier.TFN<> NULLORBLANK) AND(IEE:RP:entity.identifier.TFN<> PARENT RETURN:RP:entity.identifier.TFN)
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOGEN410115(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.RPIdentifierTFN) &&
                             !ParentDocument.RPIdentifierTFN.Equals(ChildDocument.RPIdentifierTFN);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410115",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "TFN on Schedule must match TFN on main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410115" } }
                });
            }
        }
        #endregion
        #region VR.ATO.GEN.410119
        //WHERE PARENT RETURN EXISTS
        //IF(IEE:INT:entity.identifier.ABN<> NULLORBLANK) AND(IEE:INT:entity.identifier.ABN<> PARENT RETURN:INT:entity.identifier.ABN)
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOGEN410119(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.INTIdentifierABN) &&
                             !ParentDocument.INTIdentifierABN.Equals(ChildDocument.INTIdentifierABN);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.410119",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "ABN of the Intermediary on the schedule must match the ABN on the main form",
                    Location = "/xbrli:xbrl/xbrli:context/xbrli:entity/xbrli:identifier",
                    Parameters = new ProcessMessageParameters { new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.GEN.410119" } }
                });
            }
        }
        #endregion
        #region VR.ATO.IEE.410051
        //WHERE PARENT RETURN EXISTS
        //IF([IEE49] <> NULL) AND([IEE45] <> PARENT RETURN:Report.TargetFinancial.Year)
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410051(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE49.HasValue && ChildDocument.IEE45 != ParentDocument.SMSFAR1;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410235",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Election Commencement Time must not be present if Income Year Specified is different to the parent return target financial year",
                    Location = "/xbrli:xbrl/tns:Elections.Commencement.Date",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410051" },
                        new ProcessMessageParameter { Name = "IEE49", Value = ChildDocument.IEE49.Value.ToString() }
                    }
                });
            }
        }
        #endregion
        #region VR.ATO.IEE.410070
        //WHERE PARENT RETURN EXISTS
        //IF([IEE116] = "R") AND([IEE82] <> PARENT RETURN:Report.TargetFinancial.Year)
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410070(List<ProcessMessageDocument> response)
        {
            bool assertion = ChildDocument.IEE116.ToUpper().Equals("R") && ChildDocument.IEE82.HasValue &&
                             ChildDocument.IEE82 != ParentDocument.SMSFAR1;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410070",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "The income year of the Interposed Entity Revocation must equal the income year of the main return",
                    Location = "/xbrli:xbrl/tns:Elections.Revocation.Year",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410070" },
                        new ProcessMessageParameter { Name = "IEE82", Value = ChildDocument.IEE82.Value.ToString() }
                    }
                });
            }
        }
        #endregion
        #region VR.ATO.IEE.410223
        //WHERE PARENT RETURN EXISTS
        //IF([IEE5] <> NULLORBLANK) AND([IEE5] <>
        //(PARENT RETURN:RP:tns:AddressDetails.Line1.Text WHERE ((TUPLE ELEMENT EXPLICIT tns:AddressDetails.Usage.Code = "POS") 
        //  AND(TUPLE ELEMENT EXPLICIT tns:AddressDetails.Currency.Code = "C")) IN TUPLE(xbrli\tns:AddressDetails)
        //  RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410223(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.IEE5) &&
                             !ChildDocument.IEE5.Equals(ParentDocument.SMSFAR6);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.IEE.410223",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Interposed Entity Address must be blank or be the same as on main form",
                    LongDescription = "When the IEE is lodged as a schedule, the Address of the interposed entity must be either blank or must equal the address on the main form",
                    Location = "/xbrli:xbrl/tns:AddressDetails",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410223" },
                        new ProcessMessageParameter { Name = "IEE5", Value = ChildDocument.IEE5 }
                    }
                });
            }
        }
        #endregion
        #region VR.ATO.IEE.410226
        //WHERE PARENT RETURN EXISTS
        //IF([IEE4] <> NULLORBLANK) AND([IEE4] <> 
        //(PARENT RETURN:RP:tns:OrganisationNameDetails.OrganisationalName.Text IN TUPLE (xbrli\tns:OrganisationNameDetails)
        // WHERE((TUPLE EXPLICIT tns:OrganisationNameDetails.OrganisationalNameType.Code = "MN") AND(TUPLE EXPLICIT tns:OrganisationNameDetails.Currency.Code = "C"))) 
        //    RETURN VALIDATION MESSAGE
        //ENDIF
        private void VRATOIEE410226(List<ProcessMessageDocument> response)
        {
            bool assertion = !string.IsNullOrWhiteSpace(ChildDocument.IEE4) &&
                             !ChildDocument.IEE4.Equals(ParentDocument.SMSFAR4);
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.GEN.402063",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Name on schedule does not match parent form",
                    LongDescription = "Name on schedule does not match name on parent form.",
                    Location = "/xbrli:xbrl/tns:OrganisationNameDetails.OrganisationalName.Text",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.IEE.410226" },
                        new ProcessMessageParameter { Name = "IEE4", Value = ChildDocument.IEE4 }
                    }
                });
            }
        }
        #endregion
    }
}