using System.Collections.Generic;
using DataContracts;

namespace Ato.EN.IntegrationServices.CodeGenerationSMSFAR
{
    public partial class SMSFAR2018Validator
    {
        #region VR.ATO.SMSFAR.437147

        /*  VR.ATO.SMSFAR.437147
                    Subtotal 3 must equal zero.

                Legacy Rule Format:
                    IF ([SMSFAR389] >= [SMSFAR304]) AND [SMSFAR390] <> 0
                       RETURN VALIDATION MESSAGE
                    ENDIF
                    
                    [SMSFAR304] = SMSFAR:RP:rvctc1.02.07:IncomeTax.SubtotalTaxPayable.Amount
                    [SMSFAR389] = SMSFAR:RP:rvctc1.02.08:IncomeTax.TaxOffsetNonRefundableCarryForward.Amount
                    [SMSFAR390] = SMSFAR:RP:rvctc1.02.08:IncomeTax.TaxOffsetCarriedForwardSubTotal.Amount

            Technical Business Rule Format:
                    (^SMSFAR389 >= ^SMSFAR304) AND ^SMSFAR390 <> 0

        Data Elements:
            
                    RP:^SMSFAR390 = tns:IncomeTax.TaxOffsetCarriedForwardSubTotal.Amount
            
                    RP:^SMSFAR304 = tns:IncomeTax.SubtotalTaxPayable.Amount
            
                    RP:^SMSFAR389 = tns:IncomeTax.TaxOffsetNonRefundableCarryForward.Amount
                    */
        protected void VRATOSMSFAR437147(List<ProcessMessageDocument> response, SMSFAR2018 report)
        {
            bool assertion = report.SMSFAR389.GetValueOrDefault() >= report.SMSFAR304.GetValueOrDefault() &&
                             report.SMSFAR390.GetValueOrDefault() != 0;
            if (assertion)
            {
                response.Add(new ProcessMessageDocument
                {
                    Code = "CMN.ATO.SMSFAR.437147",
                    Severity = ProcessMessageSeverity.Error,
                    Description = "Subtotal 3 must equal zero.",
                    LongDescription = "Where non-refundable carry forward tax offsets is greater than or equal to Subtotal 2, Subtotal 3 must equal zero.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.TaxOffsetCarriedForwardSubTotal.Amount",
                    Parameters = new ProcessMessageParameters
                    {
                        new ProcessMessageParameter { Name = "RuleIdentifier", Value = "VR.ATO.SMSFAR.437147" },
                        new ProcessMessageParameter { Name = "SMSFAR389", Value = report.SMSFAR389.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "SMSFAR304", Value = report.SMSFAR304.GetValueOrDefault().ToString() },
                        new ProcessMessageParameter { Name = "SMSFAR390", Value = report.SMSFAR390.GetValueOrDefault().ToString() }

                    }
                });
            }
        }
        #endregion // VR.ATO.SMSFAR.437147
    }
}