﻿using System;
using System.Collections.Immutable;
using Ato.CD.Inbound.Shared;
using VaTS;
using Ato.EN.IntegrationServices.CodeGenerationTRTAMI;

namespace Ato.CD.Inbound.TRTAMI201802
{
    internal class CrossFormLocator
    {
        private static readonly ImmutableDictionary<string, Func<TRTAMI2018, BusinessDocument, ICrossFormValidator>> CrossFormValidators;

        static CrossFormLocator()
        {
            var builder = ImmutableDictionary.CreateBuilder<string, Func<TRTAMI2018, BusinessDocument, ICrossFormValidator>>();

            builder.Add(Schedule.CGTS.ToString(), (report, schedule) => new CrossFormValidatorCGTS(report, schedule));
            builder.Add(Schedule.PSS.ToString(), (report, schedule) => new CrossFormValidatorPSS(report, schedule));
            builder.Add(Schedule.RS.ToString(), (report, schedule) => new CrossFormValidatorRS(report, schedule));
            builder.Add(Schedule.IDS.ToString(), (report, schedule) => new CrossFormValidatorIDS(report, schedule));
            builder.Add(Schedule.TRTAMIS.ToString(), (report, schedule) => new CrossFormValidatorTRTAMIS(report, schedule));

            CrossFormValidators = builder.ToImmutable();
        }

        internal static ICrossFormValidator LocateCrossFormValidatorFor(BusinessDocument schedule, TRTAMI2018 report)
            => CrossFormValidators[schedule.DocumentName](report, schedule);
    }
}